/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: ampcvReg.c,v 5.1.2.1 1999/06/30 08:33:19 matthias Exp $
 *
 */

/*
 * Defines:
 */

/* #define DEBUG_ICPCI_REG */

#ifdef DEBUG_ICPCI
#  define DEBUG_ICPCI_PLL
#endif
#ifdef DEBUG_ICPCI_PLL
#  define ICP_PLL_PRINTF(fmt,args...) icp_irq_printf (fmt, ## args)
#else
#  define ICP_PLL_PRINTF(fmt,args...)
#endif

#define READ_HANDLER_BIT(subfield, reg_name) \
	inline bit \
	pv_get_ ## subfield (u_char board_num) \
	{ \
	  pv_ ## reg_name ## _t tmp_val; \
	  tmp_val.all = icp_readw (&pv_reg[board_num]-> ## reg_name ## .all); \
	  return (tmp_val.b. ## subfield); \
        }
				 
#define READ_HANDLER_FIELD(subfield, reg_name) \
	inline u_char \
	pv_get_ ## subfield (u_char board_num) \
	{ \
	  pv_ ## reg_name ## _t tmp_val; \
	  tmp_val.all = icp_readw (&pv_reg[board_num]-> ## reg_name ## .all); \
	  return (tmp_val.b. ## subfield); \
        }
				 
#define WRITE_HANDLER_BIT(subfield, reg_name) \
	inline retval \
	pv_set_ ## subfield (u_char board_num, bit bit_value) \
	{ \
	  pv_ ## reg_name ## _t tmp_val; \
	  tmp_val.all = icp_readw (&pv_reg[board_num]-> ## reg_name ## .all); \
          tmp_val.b. ## subfield = bit_value; \
	  icp_writew (tmp_val.all, &pv_reg[board_num]-> ## reg_name ## .all); \
	  return (OK); \
        }
				 
/*
 * Includes:
 */

#include "icpciOS.h"	/* os specific stuff */
#include "icpciExt.h"	/* offical driver interface */
#include "icpciInt.h"	/* driver internals */
#include "ampcvReg.h"	/* register definitions PCVision */

/*
 * Globals:
 */

extern struct 
icp_global_t    icp;	/* main driver structure */

#ifdef DEBUG_ICPCI
const char  *pv_reg_name[] = { "modid  ",
			       "error  ",
			       "error  ",
			       "error  ",
			       "lutaddr",
			       "lutdata",
			       "adcdata",
			       "error  ",
			       "htotal ",
			       "hsyncpw",
			       "hequipw",
			       "hserrpw",
			       "vtotal ",
			       "vsyncpw",
			       "vgstart",
			       "vgwidth",
			       "hoffset",
			       "hactive",
			       "voffset",
			       "vactive",
			       "strbdly",
			       "frstdly",
			       "con1   ",
			       "con2   ",
			       "trigcon",
			       "error  ",
			       "p_clamp",
			       "error  ",
			       "error  ",
			       "error  ",
			       "error  ",
			       "error  ",
			       /* ---- */
			       "pllreg0",
			       "pllreg1",
			       "pllreg2",
			       "pllreg3",
			       "pllreg4",
			       "pllreg5",
			       "pllreg6",
			       "pllreg7",
			       /* ---- */
			       "negref "
			       "posref ",
};
#endif /* DEBUG_ICPCI */

/*
 * Prototypes:
 */

static u_short
pv_read_pllreg (u_char board, u_short addr);
static retval
pv_write_pllreg (u_char board, u_char addr, u_short value);
static retval
pv_set_lutaddr (u_char, u_char);

/*
 * Read/Write Functions for Clock Generator
 * for Details on Operation see Data Sheet ICS1522
 * from Integrated Circuit Systems Inc.
 */

static u_short
pv_read_pllreg (u_char board, u_short addr) {

  int count;

  u_char  dummy = 0;
  u_short tmp_val = 0;

  if (addr > PV_PLLADDR_MASK - 1) {
    icp_printf ("icpci[%c]: error in pv_read_pllreg - addr %d > %d \n",
		'E', addr, PV_PLLADDR_MASK - 1);
    return (0);
  }

  /* write address */
  ICP_PLL_PRINTF ("READ  I: ");
  icp_writew (icp_readw (&pv_reg[board]->con2.all) & ~1,
	      &pv_reg[board]->con2.all); /* PV_PLL_PROG */
  ICP_PLL_PRINTF ("%c ", (icp_readw (&pv_reg[board]->con2.all) & 1) ?
		  'H' : 'L');
  icp_writew (PV_PLL_READ, &pv_reg[board]->pllprg.all);
  ICP_PLL_PRINTF ("%d ", PV_PLL_READ);
  for (count = 0; count < 14; count++) {
    dummy = (addr >> count) & 1;
    ICP_PLL_PRINTF ("%d ", dummy);
    icp_writew (dummy, &pv_reg[board]->pllprg.all);
  }
  icp_writew (icp_readw (&pv_reg[board]->con2.all) | 1,
	      &pv_reg[board]->con2.all ); /* PLL_PLL_NORM */
  ICP_PLL_PRINTF ("%c ", (PV_PLL_NORM) ? 'H' : 'L');
  icp_writew (0, &pv_reg[board]->pllprg.all);
  ICP_PLL_PRINTF ("%d.\n", dummy);

  /* read value */
  ICP_PLL_PRINTF ("READ II: ");
  icp_writew (icp_readw (&pv_reg[board]->con2.all) & ~1,
	      &pv_reg[board]->con2.all); /* PV_PLL_PROG */
  ICP_PLL_PRINTF ("%c ", (icp_readw (&pv_reg[board]->con2.all) & 1) ?
		  'H' : 'L');
  for (count = 0; count < 14; count++) {
    dummy = icp_readw (&pv_reg[board]->pllprg.all) & 1;
    ICP_PLL_PRINTF ("%d ", dummy);
    tmp_val |= dummy << count;
  }
  dummy = icp_readw (&pv_reg[board]->pllprg.all) & 1;
  ICP_PLL_PRINTF ("%d ", dummy);
  icp_writew (icp_readw (&pv_reg[board]->con2.all) | 1,
	      &pv_reg[board]->con2.all ); /* PLL_PLL_NORM */
  ICP_PLL_PRINTF ("%c ", (PV_PLL_NORM) ? 'H' : 'L');
  icp_writew (0, &pv_reg[board]->pllprg.all);
  ICP_PLL_PRINTF ("0.\n");   
  
  if ((tmp_val & PV_PLLADDR_MASK) != addr) {
    icp_printf ("icpci[%c]: error in pv_read_pllreg - addr %d != %d\n",
		'E', (tmp_val & PV_PLLADDR_MASK), addr);
    return (0);
  }

  return (tmp_val >> PV_PLLDATA_SHIFT);
}

static retval
pv_write_pllreg (u_char board, u_char addr, u_short value) {

  int count;

  u_char  dummy;
  u_short tmp_val = 0;

  if (addr > PV_PLLADDR_MASK - 1) {
    icp_printf ("icpci[%c]: error in pv_write_pllreg - addr %d > %d \n",
		'E', addr, PV_PLLADDR_MASK - 1);
    return (ERROR);
  }

  if (value & ~PV_PLLDATA_MASK) {
    icp_printf ("icpci[%c]: error in pv_write_pllreg - value %d > %d \n",
		'E', addr, PV_PLLDATA_MASK);
    return (ERROR);
  }

  tmp_val = addr | ((value & PV_PLLDATA_MASK) << PV_PLLDATA_SHIFT);

  /* write combined address/data */
  ICP_PLL_PRINTF ("WRITE I: ");
  icp_writew (icp_readw (&pv_reg[board]->con2.all) & ~1,
	      &pv_reg[board]->con2.all); /* PV_PLL_PROG */
  ICP_PLL_PRINTF ("%c ", (icp_readw (&pv_reg[board]->con2.all) & 1) ?
		  'H' : 'L');
  icp_writew (PV_PLL_WRITE, &pv_reg[board]->pllprg.all);  
  ICP_PLL_PRINTF ("%d ", PV_PLL_WRITE);
  for (count = 0; count < 14; count++) {
    dummy = (tmp_val >> count) & 1;
    ICP_PLL_PRINTF ("%d ", dummy);
    icp_writew (dummy, &pv_reg[board]->pllprg.all);
  }
  icp_writew (icp_readw (&pv_reg[board]->con2.all) | 1,
	      &pv_reg[board]->con2.all ); /* PLL_PLL_NORM */
  ICP_PLL_PRINTF ("%c ", (icp_readw (&pv_reg[board]->con2.all) & 1) ?
		  'H' : 'L');
  icp_writew (0, &pv_reg[board]->pllprg.all);  
  ICP_PLL_PRINTF ("0.\n");

  return (OK);
}

/*
 * Special Read Handler for PCVision Registers
 */

READ_HANDLER_BIT(hsyncpol, vsyncpw)
READ_HANDLER_BIT(vsyncpol, vsyncpw)
READ_HANDLER_BIT(lenpol, strbdly)
READ_HANDLER_BIT(fenpol, strbdly)
READ_HANDLER_BIT(frstonv, frstdly)
READ_HANDLER_BIT(frstmd, frstdly)
READ_HANDLER_BIT(frstpol, frstdly)
READ_HANDLER_BIT(frstsz, frstdly)
READ_HANDLER_BIT(syncsel, con1);
READ_HANDLER_BIT(chsyncen, con1);
READ_HANDLER_BIT(vsyncen, con1);
READ_HANDLER_BIT(clksrc, con1);
READ_HANDLER_BIT(lpfsel, con1);
READ_HANDLER_BIT(vscan, con1);
READ_HANDLER_BIT(iscale, con1);
READ_HANDLER_BIT(vclkpol, con1);
READ_HANDLER_BIT(smode, con1);
READ_HANDLER_BIT(fldshft, con1);
READ_HANDLER_BIT(einpol, con2);
READ_HANDLER_BIT(ein0ien, con2);
READ_HANDLER_BIT(ein1ien, con2);
READ_HANDLER_BIT(ein0, con2);
READ_HANDLER_BIT(ein1, con2);
READ_HANDLER_BIT(eout0, con2);
READ_HANDLER_BIT(eout1, con2);
READ_HANDLER_BIT(sotint, con2);
READ_HANDLER_BIT(eotint, con2);
READ_HANDLER_BIT(ein0int, con2);
READ_HANDLER_BIT(ein1int, con2);
READ_HANDLER_BIT(trigen, trigcon)
READ_HANDLER_BIT(trigcy, trigcon)
READ_HANDLER_BIT(sotien, trigcon)
READ_HANDLER_BIT(eotien, trigcon)
READ_HANDLER_BIT(trigsrc, trigcon)
READ_HANDLER_BIT(trigpol, trigcon)
READ_HANDLER_BIT(strbmd, trigcon)
READ_HANDLER_BIT(strben, trigcon)
READ_HANDLER_BIT(strbpol, trigcon)
READ_HANDLER_BIT(skpfldmd, trigcon)
READ_HANDLER_BIT(pclmpol, p_clamp)
READ_HANDLER_BIT(clmpsrc, p_clamp)

READ_HANDLER_FIELD(rev, modid)
READ_HANDLER_FIELD(camsel, con1);

u_char
pv_get_lutdata (u_char board, struct pv_lut_t *get_lut) {
  
  int count;
  
  if (pv_set_lutaddr (board, PV_LUTADDR_BASE) < 0) return (ERROR);
  
  for (count = PV_MIN_LUTDATA; count <= PV_MAX_LUTDATA; count++)
    get_lut->lutdata[count] = icp_readw (&pv_reg[board]->lutdata);
  
  return (OK);
}

u_char
pv_get_pref (u_char board) {

  if (pv_set_lutaddr (board, PV_LUTADDR_PREF) < 0)
    icp_error (board, "pv_set_lutaddr");

  return (icp_readw (&pv_reg[board]->adcdata));
}

u_char
pv_get_nref (u_char board) {

  if (pv_set_lutaddr (board, PV_LUTADDR_NREF) < 0)
      icp_error (board, "pv_set_lutaddr");
  
  return (icp_readw (&pv_reg[board]->adcdata));
}

u_short
pv_get_htotal (u_char board) {

  return ((icp_readw (&pv_reg[board]->htotal) << 1) + 1);
}

u_char
pv_get_hsyncpw (u_char board) {

  return (icp_readw (&pv_reg[board]->hsyncpw) + 1);
}  

u_char
pv_get_hequipw (u_char board) {

  return (icp_readw (&pv_reg[board]->hequipw) + 1);
}  

u_short
pv_get_hserrpw (u_char board) {

  return (icp_readw (&pv_reg[board]->hserrpw) + 1);
}  

u_short
pv_get_vtotal (u_char board) {
  
  return (icp_readw (&pv_reg[board]->vtotal) + 1);
}  

u_char
pv_get_vsyncpw (u_char board) {
  
  pv_vsyncpw_t tmp_val;

  tmp_val.all = icp_readw (&pv_reg[board]->vsyncpw.all);
  return (++tmp_val.b.vsyncpw);
}  

u_short
pv_get_vgstart (u_char board) {
  
  return (icp_readw (&pv_reg[board]->vgstart) + 1);
}  

u_char
pv_get_vgwidth (u_char board) {

  return (icp_readw (&pv_reg[board]->vgwidth) + 1);
}  

u_short
pv_get_hoffset (u_char board) {

  return ((icp_readw (&pv_reg[board]->hoffset) & PV_HOFFSET_MASK) + 1);
}  

u_short
pv_get_hactive (u_char board) {

  return ((icp_readw (&pv_reg[board]->hactive) & PV_HACTIVE_MASK) + 1);
}  

u_short
pv_get_voffset (u_char board) {

  return ((icp_readw (&pv_reg[board]->voffset) & PV_VOFFSET_MASK) + 1);
}  

u_short
pv_get_vactive (u_char board) {

  return ((icp_readw (&pv_reg[board]->vactive) & PV_VACTIVE_MASK) + 1);
}  

u_char
pv_get_strbdly (u_char board) {

  pv_strbdly_t tmp_val;

  tmp_val.all = icp_readw (&pv_reg[board]->strbdly.all);
  return (++tmp_val.b.strbdly);
}  

u_char
pv_get_froff (u_char board) {

  pv_frstdly_t tmp_val;

  tmp_val.all = icp_readw (&pv_reg[board]->frstdly.all);
  return (++tmp_val.b.froff);
}  

u_char
pv_get_ccnt (u_char board) {

  pv_p_clamp_t tmp_val;

  tmp_val.all = icp_readw (&pv_reg[board]->p_clamp.all);
  return (tmp_val.b.ccnt << 1);
}

u_short
pv_get_fdiv (u_char board) {

  return ((pv_read_pllreg (board, PV_PLLREG0_ADDR) & PV_FBKDIV_MASK) + 1);
}

u_short
pv_get_rdiv (u_char board) {

  return ((pv_read_pllreg (board, PV_PLLREG3_ADDR) & PV_REFDIV_MASK) + 1);
}

u_char
pv_get_vcogain (u_char board) {

  return (pv_read_pllreg (board, PV_PLLREG4_ADDR) & PV_VCO_MASK);
}

u_char
pv_get_pda (u_char board) {

  return ((pv_read_pllreg (board, PV_PLLREG5_ADDR) & PV_PDA_MASK) >>
	  PV_PDA_SHIFT);
}

u_char
pv_get_lcnt (u_char board) {

  return (pv_read_pllreg (board, PV_PLLREG6_ADDR) & PV_LCNT_MASK);  
}

/*
 * Special Write Handler for PCVision Registers
 */

WRITE_HANDLER_BIT(hsyncpol, vsyncpw)
WRITE_HANDLER_BIT(vsyncpol, vsyncpw)
WRITE_HANDLER_BIT(frstonv, frstdly)
WRITE_HANDLER_BIT(frstmd, frstdly)
WRITE_HANDLER_BIT(frstpol, frstdly)
WRITE_HANDLER_BIT(frstsz, frstdly)
WRITE_HANDLER_BIT(syncsel, con1);
WRITE_HANDLER_BIT(chsyncen, con1);
WRITE_HANDLER_BIT(vsyncen, con1);
WRITE_HANDLER_BIT(clksrc, con1);
WRITE_HANDLER_BIT(lpfsel, con1);
WRITE_HANDLER_BIT(iscale, con1);
WRITE_HANDLER_BIT(smode, con1);
WRITE_HANDLER_BIT(fldshft, con1);
WRITE_HANDLER_BIT(einpol, con2);
WRITE_HANDLER_BIT(ein0ien, con2);
WRITE_HANDLER_BIT(ein1ien, con2);
WRITE_HANDLER_BIT(eout0, con2);
WRITE_HANDLER_BIT(eout1, con2);
WRITE_HANDLER_BIT(trigen, trigcon)
WRITE_HANDLER_BIT(sotien, trigcon)
WRITE_HANDLER_BIT(eotien, trigcon)
WRITE_HANDLER_BIT(trigsrc, trigcon)
WRITE_HANDLER_BIT(trigpol, trigcon)
WRITE_HANDLER_BIT(strbmd, trigcon)
WRITE_HANDLER_BIT(strbpol, trigcon)
WRITE_HANDLER_BIT(skpfldmd, trigcon)
WRITE_HANDLER_BIT(pclmpol, p_clamp)
WRITE_HANDLER_BIT(clmpsrc, p_clamp)

static retval
pv_set_lutaddr (u_char board, u_char value) {

  switch (value) {
  case PV_LUTADDR_CONT:
    icp_writew (PV_LUTADDR_CONT, &pv_reg[board]->lutaddr);
    return (OK);
  case PV_LUTADDR_NREF:
    icp_writew (PV_LUTADDR_NREF, &pv_reg[board]->lutaddr);
    return (OK);
  case PV_LUTADDR_PREF:
    icp_writew (PV_LUTADDR_PREF, &pv_reg[board]->lutaddr);
    return (OK);
  }

  icp.errno[board] = -EICP_NOSUPPORT;
  return (ERROR);
}

int
pv_set_lutdata (u_char board, struct pv_lut_t *set_lut) {

  int count;

  if (set_lut == NULL) {
    icp.errno[board] = -EICP_POINTER;
    return (ERROR);
  }

  if (pv_set_lutaddr (board, PV_LUTADDR_BASE) < 0) return (ERROR);

  for (count = PV_MIN_LUTDATA; count <= PV_MAX_LUTDATA; count++)
    icp_writew (set_lut->lutdata[count], &pv_reg[board]->lutdata);

  return (OK);
}

retval
pv_set_pref (u_char board, u_char value) {

  if (pv_set_lutaddr (board, PV_LUTADDR_PREF) < 0) return (ERROR);

  if (icp.pci[board]->revision < PV_REV_B)
    icp_writew (value, &pv_reg[board]->adcdata);
  else
    icp_writew (value & PV_PREF_MASK, &pv_reg[board]->adcdata);
  
  return (OK);
}

retval
pv_set_nref (u_char board, u_char value) {

  if (pv_set_lutaddr (board, PV_LUTADDR_NREF) < 0) return (ERROR);
  
  if (icp.pci[board]->revision < PV_REV_B)
    icp_writew (value, &pv_reg[board]->adcdata);
  else
    icp_writew (value & PV_NREF_MASK, &pv_reg[board]->adcdata);
  
  return (OK);
}

retval
pv_set_htotal (u_char board, u_short value) {

  if (value < PV_MIN_HTOTAL || value > PV_MAX_HTOTAL) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (((value >> 1) - 1) & PV_HTOTAL_MASK, &pv_reg[board]->htotal);

  return (OK);
}

retval
pv_set_hsyncpw (u_char board, u_char value) {

  if (value < PV_MIN_HSYNCPW || value > PV_MAX_HSYNCPW) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_HSYNCPW_MASK, &pv_reg[board]->hsyncpw);

  return (OK);
}  

retval
pv_set_hequipw (u_char board, u_char value) {

  if (value < PV_MIN_HEQUIPW || value > PV_MAX_HEQUIPW) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_HEQUIPW_MASK, &pv_reg[board]->hequipw);

  return (OK);
}  

retval
pv_set_hserrpw (u_char board, u_short value) {

  if (value < PV_MIN_HSERRPW || value > PV_MAX_HSERRPW) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_HSERRPW_MASK, &pv_reg[board]->hserrpw);

  return (OK);
}  

retval
pv_set_vtotal (u_char board, u_short value) {
  
  if (value < PV_MIN_VTOTAL || value > PV_MAX_VTOTAL) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_VTOTAL_MASK, &pv_reg[board]->vtotal);
  
  return (OK);
}  

retval
pv_set_vsyncpw (u_char board, u_char value) {

  pv_vsyncpw_t tmp_val;

  if (value < PV_MIN_VSYNCPW || value > PV_MAX_VSYNCPW) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  tmp_val.all = icp_readw (&pv_reg[board]->vsyncpw.all);
  tmp_val.b.vsyncpw = --value;
  icp_writew (tmp_val.all, &pv_reg[board]->vsyncpw.all);

  return (OK);
}  

retval
pv_set_vgstart (u_char board, u_short value) {
  
  if (value < PV_MIN_VGSTART || value > PV_MAX_VGSTART) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_VGSTART_MASK, &pv_reg[board]->vgstart);

  return (OK);
}  

retval
pv_set_vgwidth (u_char board, u_char value) {
  
  if (value < PV_MIN_VGWIDTH || value > PV_MAX_VGWIDTH) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_VGWIDTH_MASK, &pv_reg[board]->vgwidth);

  return (OK);
}  

retval
pv_set_hoffset (u_char board, u_short value) {

  if (value < PV_MIN_HOFFSET || value > PV_MAX_HOFFSET) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_HOFFSET_MASK, &pv_reg[board]->hoffset);

  return (OK);
}  

retval
pv_set_hactive (u_char board, u_short value) {

  if (value < PV_MIN_HACTIVE || value > PV_MAX_HACTIVE) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_HACTIVE_MASK, &pv_reg[board]->hactive);

  return (OK);
}  

retval
pv_set_voffset (u_char board, u_short value) {

  if (value < PV_MIN_VOFFSET || value > PV_MAX_VOFFSET) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  icp_writew (--value & PV_VOFFSET_MASK, &pv_reg[board]->voffset);

  return (OK);
}  

retval
pv_set_vactive (u_char board, u_short value) {

  if (icp.pci[board]->revision < PV_REV_B) {
    if (value < PV_MIN_VACTIVE || value > PV_MAX_VACTIVEA) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
  } else {
    if (value < PV_MIN_VACTIVE || value > PV_MAX_VACTIVEB) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
  }
  icp_writew (--value & PV_VACTIVE_MASK, &pv_reg[board]->vactive);

  return (OK);
}  

retval
pv_set_strbdly (u_char board, u_char value) {

  pv_strbdly_t tmp_val;

  if (value < PV_MIN_STRBDLY || value > PV_MAX_STRBDLY) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  
  tmp_val.all = icp_readw (&pv_reg[board]->strbdly.all);
  tmp_val.b.strbdly = --value & PV_STRBDLY_MASK;
  icp_writew (tmp_val.all, &pv_reg[board]->strbdly.all);

  return (OK);
}  

retval
pv_set_lenpol (u_char board, bit value) {

  pv_strbdly_t tmp_val;

  if (icp.pci[board]->revision < PV_REV_B) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = icp_readw (&pv_reg[board]->strbdly.all);
  tmp_val.b.lenpol = value;
  icp_writew (tmp_val.all, &pv_reg[board]->strbdly.all);

  return (OK);
}

retval
pv_set_fenpol (u_char board, bit value) {

  pv_strbdly_t tmp_val;
  
  if (icp.pci[board]->revision < PV_REV_B) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = icp_readw (&pv_reg[board]->strbdly.all);
  tmp_val.b.fenpol = value;
  icp_writew (tmp_val.all, &pv_reg[board]->strbdly.all);

  return (OK);
}

retval
pv_set_froff (u_char board, u_char value) {

  pv_frstdly_t tmp_val;

  if (icp.pci[board]->revision < PV_REV_B) {
    if (value < PV_MIN_FROFF || value > PV_MAX_FROFFA) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
  } else {
    if (value < PV_MIN_FROFF || value > PV_MAX_FROFFB) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
  }

  tmp_val.all = icp_readw (&pv_reg[board]->frstdly.all);
  tmp_val.b.froff = --value & PV_FROFF_MASK;
  icp_writew (tmp_val.all, &pv_reg[board]->frstdly.all);

  return (OK);
}  

retval
pv_set_vscan (u_char board, bit value) {

  pv_con1_t tmp_val;

  if (icp.pci[board]->revision < PV_REV_B) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = icp_readw (&pv_reg[board]->con1.all);
  tmp_val.b.vscan = value;
  icp_writew (tmp_val.all, &pv_reg[board]->con1.all);

  return (OK);
}

retval
pv_set_vclkpol (u_char board, bit value) {

  pv_con1_t tmp_val;

  if (icp.pci[board]->revision < PV_REV_B) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = icp_readw (&pv_reg[board]->con1.all);
  tmp_val.b.vclkpol = value;
  icp_writew (tmp_val.all, &pv_reg[board]->con1.all);

  return (OK);
}

retval
pv_set_camsel (u_char board, u_char value) {

  pv_con1_t tmp_val;

  switch (value) {
  case PV_CAM_0:
  case PV_CAM_1:
  case PV_CAM_2:
  case PV_CAM_3:
    break;
  default:
  icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  tmp_val.all = icp_readw (&pv_reg[board]->con1.all);
  tmp_val.b.camsel = value;
  icp_writew (tmp_val.all, &pv_reg[board]->con1.all);

  return (OK);
}

retval
pv_set_strben (u_char board, bit value) {

  pv_trigcon_t tmp_val;

  if (icp.pci[board]->revision < PV_REV_B) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = icp_readw (&pv_reg[board]->trigcon.all);
  tmp_val.b.strben = value;
  icp_writew (tmp_val.all, &pv_reg[board]->trigcon.all);

  return (OK);
}

retval
pv_set_ccnt (u_char board, u_char value) {

  pv_p_clamp_t tmp_val;

  if (value > PV_MAX_CCNT) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  tmp_val.all = icp_readw (&pv_reg[board]->p_clamp.all);
  tmp_val.b.ccnt = value >> 1;
  icp_writew (tmp_val.all, &pv_reg[board]->p_clamp.all);

  return (OK);
}
  
retval
pv_set_fdiv (u_char board, u_short value) {

  if (value < PV_FBKDIV_MIN || value > PV_FBKDIV_MAX) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  pv_write_pllreg (board, PV_PLLREG0_ADDR, --value & PV_FBKDIV_MASK);

  return (OK);
}

retval
pv_set_rdiv (u_char board, u_short value) {

  u_short tmp_val;

  if (value < PV_REFDIV_MIN || value > PV_REFDIV_MAX) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_REFDIV_INIT & ~PV_REFDIV_MASK) | --value;
  pv_write_pllreg (board, PV_PLLREG3_ADDR, tmp_val);

  return (OK);
}

retval
pv_set_vcogain (u_char board, u_char value) {

  u_short tmp_val;

  if (value > PV_VCO_MASK) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_PLLREG4_INIT & ~PV_VCO_MASK) | value;
  pv_write_pllreg (board, PV_PLLREG4_ADDR, tmp_val);

  return (OK);
}

retval
pv_set_pda (u_char board, u_char value) {

  u_short tmp_val;

  value <<= PV_PDA_SHIFT;

  if (value > PV_PDA_MASK) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_PLLREG5_INIT & ~(PV_PDA_MASK << PV_PDA_SHIFT)) | value;
  pv_write_pllreg (board, PV_PLLREG5_ADDR, tmp_val);

  return (OK);
}

retval
pv_set_lcnt (u_char board, u_char value) {

  u_short tmp_val;

  if (value > PV_LCNT_MASK) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_PLLREG6_INIT & ~PV_LCNT_MASK) | value;
  pv_write_pllreg (board, PV_PLLREG6_ADDR, tmp_val);

  return (OK);
}

/*
 * Special Initialization Handler for PCVision Registers
 */

retval
pv_init_adccon (u_char board) {

  if (pv_set_lutaddr (board, PV_LUTADDR_CONT) < 0) return (ERROR);
  icp_writew (PV_RES_INIT, &pv_reg[board]->adcdata);

  return (OK);
}

retval
pv_init_vsyncpw (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->vsyncpw.all);
  return (OK);
}
  
retval
pv_init_strbdly (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->strbdly.all);
  return (OK);
}
  
retval
pv_init_frstdly (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->frstdly.all);
  return (OK);
}
  
retval
pv_init_con1 (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->con1.all);
  return (OK);
}

retval
pv_init_con2 (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->con2.all);
  return (OK);
}

retval
pv_init_trigcon (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->trigcon.all);
  return (OK);
}

retval
pv_init_p_clamp (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->p_clamp.all);
  return (OK);
}

retval
pv_init_pllregs (u_char board) {  

  if (pv_write_pllreg (board, PV_PLLREG0_ADDR, PV_FBKDIV_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (board, PV_PLLREG1_ADDR, PV_PLLREG1_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (board, PV_PLLREG2_ADDR, PV_PLLREG2_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (board, PV_PLLREG3_ADDR, PV_REFDIV_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (board, PV_PLLREG4_ADDR, PV_PLLREG4_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (board, PV_PLLREG5_ADDR, PV_PLLREG5_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (board, PV_PLLREG6_ADDR, PV_PLLREG6_INIT) < 0)
    return (ERROR);

  return (OK);
}
/* 
 * Miscellanous functions for PCVision
 */
 
/* trigger per software */
int
pv_sw_trig (u_char board) {

  icp_writew (PV_RES_INIT, &pv_reg[board]->sw_trig);
  return (OK);
}

/* linearize lookup table */
int
pv_lin_lut (u_char board) {

  int count;

  if (pv_set_lutaddr (board, PV_LUTADDR_BASE) < 0) return (ERROR);

  for (count = PV_MIN_LUTDATA; count <= PV_MAX_LUTDATA; count++)
    icp_writew (count, &pv_reg[board]->lutdata);

  return (OK);
}

#ifdef DEBUG_ICPCI
/* make register dump */
void
pv_dump_regs (u_char board) {
  
  int count1;
  int count2;

  u_short *reg_ptr = (u_short*)pv_reg[board];
  
  for (count1 = 0; count1 < PV_REG_SIZE; count1++, reg_ptr++)
    if (strcmp (pv_reg_name[count1], "error  "))
      icp_printf ("icpci[%d]: %s %#04x\n", board,
		  pv_reg_name[count1], icp_readw (reg_ptr));

  for (count2 = 0; count2 < PV_PLL_SIZE; count1++, count2++)
    icp_printf ("icpci[%d]: %s %#04x\n", board,
		pv_reg_name[count1],
		pv_read_pllreg (board, count2));

  icp_printf ("icpci[%d]: %s %#04x\n", board, pv_reg_name[count1++],
	      pv_get_nref(board));
  icp_printf ("icpci[%d]: %s %#04x\n", board, pv_reg_name[count1],
	      pv_get_pref(board));

}
#endif
