/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: extime_icpci.c,v 5.1.2.1 1999/06/03 08:57:43 matthias Exp $
 *
 */

/*
 * Defines:
 */

#define	MIN_EXTIME	ICP_DLY_MIN
#define	MAX_EXTIME	ICP_DLY_MAX
#define	DEF_EXTIME	(279 * (ICP_DLY_BASE << 11))	/* approx. 40ms */

/*
 * Includes:
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <termios.h>
#include <getopt.h>

#include "icpciExt.h"

/*
 * Globals:
 */

static const char *dg_camera_device[] =
{
  "/dev/ic0idI",
  "/dev/ic1idI",
  "/dev/ic2idI",
  "/dev/ic3idI"
};

static struct termios save_tty_state;
static void restore_tty_state (void);

extern char *optarg;
extern int optind, opterr, optopt;

int dev_fd = -1;

/*
 * Prototypes:
 */

void
restore_tty_state (void) {
  
  tcsetattr (0, TCSAFLUSH, &save_tty_state);
  printf ("\n");
}

int
main (int argc, char *argv[]) {
  
  char interactive = TRUE;
  char command, argument;
  char range = '5';	/* adjust to DEF_EXTIME! */
  int board = 0;
  
  long extime = DEF_EXTIME;
  long new_extime = DEF_EXTIME;

  struct termios set_tty_state;

  struct timespec extimestr;

  while (1)
    {
    if ((argument = getopt (argc, argv, "b:e:?")) == -1)
      break;
    switch (argument)
      {
      case 'b':
	if (optarg)
	  {
	    board = atoi (optarg);
	    if (board < 0 || board >= ICP_BOARDS_MAX)
	      {
		printf ("no or wrong board selected (%d).\n", board);
		exit (EXIT_FAILURE);
	      }
	  }
	break;
      case 'e':
	if (optarg)
	  {
	    extime = atof (optarg) * 1000.0;
	    if (extime < MIN_EXTIME || extime > MAX_EXTIME)
	      {
		printf ("delay time of range (%07.3f us - %07.3f ms).\n",
			(float)MIN_EXTIME / 1000.0,
			(float)MAX_EXTIME / 1000000.0);
		exit (EXIT_FAILURE);
	      }
	  }
	interactive = FALSE;
	break;
      case '?':
      default:
	printf ("Usage: extime_icpci   -b board (%d) [%d - %d]\n",
		board,
		0,
		ICP_BOARDS_MAX - 1);
	printf ("                      -e delay time "
		"(%07.3f ms) [%07.3f us - %07.3f ms]\n",
		(float)extime / 1000000.0,
		(float)MIN_EXTIME / 1000.0,
		(float)MAX_EXTIME / 1000000.0);
	printf ("Interactive commands: <  - minimal value\n");
	printf ("                      -  - decrement one step\n");
	printf ("                     1-6 - select range\n");
	printf ("                      +  - increment one step\n");
	printf ("                      >  - maximal value\n");
	printf ("                      q  - exit program\n");
	exit (EXIT_FAILURE);
	break;
      }
    }

  if ((dev_fd = open (dg_camera_device[board], O_RDONLY)) < 0) {
    perror ("open camera device");
    exit (EXIT_FAILURE);
  }
  
  if (tcgetattr (0, &save_tty_state) < 0)
    {
      perror ("tcgetattr");
      exit (EXIT_FAILURE);
    }
  if (atexit (restore_tty_state) < 0)
    {
      perror ("atexit restore_tty_state");
      exit (EXIT_FAILURE);
    }
  cfmakeraw (&set_tty_state);
  if (tcsetattr (0, TCSAFLUSH, &set_tty_state) < 0)
    {
      perror ("tcsetattr");
      exit (EXIT_FAILURE);
    }
  
  extimestr.tv_sec = 0;

  if (interactive)
    {
      do
	{
	  printf ("<,-,1-6,+,>,q ");
	  if (extime < 1000000)
	    printf ("[%c|%07.3f us]: ", range, (float)extime / 1000.0);
	  else
	    printf ("[%c|%07.3f ms]: ", range, (float)extime / 1000000.0);

	  extimestr.tv_nsec = extime;
	  if (ioctl (dev_fd, GIOC_DG_SET_EXTIME, &extimestr) < 0)
	    {
	      perror ("GIOC_DG_SET_EXTIME");
	      exit (EXIT_FAILURE);
	    }
	  
	  command = getchar();
	  switch (command)
	    {
	    case '1':
	      extime = (ICP_DLY_BASE << 1) * 1025;
	      range = '1';
	      break;

	    case '2':
	      extime = (ICP_DLY_BASE << 3) * 1025;
	      range = '2';
	      break;

	    case '3':
	      extime = (ICP_DLY_BASE << 5) * 1025;
	      range = '3';
	      break;

	    case '4':
	      extime = (ICP_DLY_BASE << 7) * 1025;
	      range = '4';
	      break;

	    case '5':
	      extime = (ICP_DLY_BASE << 9) * 1025;
	      range = '5';
	      break;

	    case '6':
	      extime = (ICP_DLY_BASE << 11) * 1025;
	      range = '6';
	      break;

	    case '<':
	      extime = (ICP_DLY_BASE << 1) * 3;
	      range = '1';
	      break;

	    case '>':
	      extime = (ICP_DLY_BASE << 11) * 4098;
	      range = '6';
	      break;

	    case '+':
	      if (extime > (ICP_DLY_BASE << 9) * 4098)
		{
		  range = '6';
		  new_extime = extime + (ICP_DLY_BASE << 11);
		}
	      else
		if (extime > (ICP_DLY_BASE << 7) * 4098)
		  {
		    range = '5';
		    if ((new_extime = extime + (ICP_DLY_BASE << 9))
			> (ICP_DLY_BASE << 9) * 4098)
		      new_extime = (ICP_DLY_BASE << 11) * 1025;
		  }
		else
		  if (extime > (ICP_DLY_BASE << 5) * 4098)
		    {
		      range = '4';
		      if ((new_extime = extime + (ICP_DLY_BASE << 7))
			  > (ICP_DLY_BASE << 7) * 4098)
			new_extime = (ICP_DLY_BASE << 9) * 1025;
		    }
		  else
		    if (extime > (ICP_DLY_BASE << 3) * 4098)
		      {
			range = '3';
			if ((new_extime = extime + (ICP_DLY_BASE << 5))
			    > (ICP_DLY_BASE << 5) * 4098)
			  new_extime = (ICP_DLY_BASE << 7) * 1025;
		      }
		    else
		      if (extime > (ICP_DLY_BASE << 1) * 4098)
			{
			  range = '2';
			  if ((new_extime = extime + (ICP_DLY_BASE << 3))
			      > (ICP_DLY_BASE << 3) * 4098)
			    new_extime = (ICP_DLY_BASE << 5) * 1025;
			}
		      else
			{
			  range = '1';
			  if ((new_extime = extime + (ICP_DLY_BASE << 3))
			      > (ICP_DLY_BASE << 1) * 4098)
			    new_extime = (ICP_DLY_BASE << 3) * 1025;
			}
	      if (new_extime < MAX_EXTIME)
		extime = new_extime;
	      break;

	    case '-':
	      if (extime > (ICP_DLY_BASE << 9) * 4098)
		{
		  range = '6';
		  if ((new_extime = extime - (ICP_DLY_BASE << 11))
		      < (ICP_DLY_BASE << 11) * 1025)
		    new_extime = (ICP_DLY_BASE << 9) * 4098;
		}
	      else
		if (extime > (ICP_DLY_BASE << 7) * 4098)
		  {
		    range = '5';
		    if ((new_extime = extime - (ICP_DLY_BASE << 9))
			< (ICP_DLY_BASE << 9) * 1025)
		      new_extime = (ICP_DLY_BASE << 7) * 4098;
		  }
		else
		  if (extime > (ICP_DLY_BASE << 5) * 4098)
		    {
		      range = '4';
		      if ((new_extime = extime - (ICP_DLY_BASE << 7))
			  < (ICP_DLY_BASE << 7) * 1025)
			new_extime = (ICP_DLY_BASE << 5) * 4098;
		    }
		  else
		    if (extime > (ICP_DLY_BASE << 3) * 4098)
		      {
			range = '3';
			if ((new_extime = extime - (ICP_DLY_BASE << 5))
			    < (ICP_DLY_BASE << 5) * 1025)
			  new_extime = (ICP_DLY_BASE << 3) * 4098;
		      }
		    else
		      if (extime > (ICP_DLY_BASE << 1) * 4098)
			{
			  range = '2';
			  if ((new_extime = extime - (ICP_DLY_BASE << 3))
			      < (ICP_DLY_BASE << 3) * 1025)
			    new_extime = (ICP_DLY_BASE << 1) * 4098;
			}
		      else
			{
			  range = '1';
			  new_extime = extime - (ICP_DLY_BASE << 1);
			}
	      if (new_extime > MIN_EXTIME)
		extime = new_extime;
	      break;

	    case 'q':
	      if (dev_fd != -1)
		close (dev_fd);
	      exit (OK);
	      break;

	    default:
	      break;
	    }

	  printf ("\r");
	} while (1);
    }
  else
    {
      extimestr.tv_nsec = extime;
      if (ioctl (dev_fd, GIOC_DG_SET_EXTIME, &extime) < 0)
	{
	  perror ("GIOC_DG_SET_EXTIME");
	  exit (EXIT_FAILURE);
	}
    }
  
  if (dev_fd != -1)
    close (dev_fd);
  
  exit (EXIT_SUCCESS);
}

